/* Copyright 2022 Sensaru GmbH */

#ifndef LIBC1_NET_SRC_PROXYPROTOCOL_H_
#define LIBC1_NET_SRC_PROXYPROTOCOL_H_

#include "Exceptions.h"
#include "Socket.h"

#include <cstdint>
#include <array>

namespace C1Net {

class ProxyProtocolException : public Exception {
 public:
  explicit ProxyProtocolException(const std::string &message) : Exception(message) {}
};

class ProxyProtocol {
 public:
  std::string GetIpAddress();
  uint16_t GetPort();

  /**
   * Returns `true` when a new IP address was found and `false` when the local connection address should be kept.
   *
   * @param socket
   * @return
   */
  bool Process(PSocket &socket);
 private:
  static constexpr std::array<uint8_t, 12> kV2Signature{0x0D, 0x0A, 0x0D, 0x0A, 0, 0x0D, 0x0A, 0x51, 0x55, 0x49, 0x54, 0x0A};

  union {
    struct {
      char line[108];
    } v1;
    struct {
      uint8_t sig[12];
      uint8_t ver_cmd;
      uint8_t fam;
      uint16_t len;
      union {
        struct {  /* for TCP/UDP over IPv4, len = 12 */
          uint32_t src_addr;
          uint32_t dst_addr;
          uint16_t src_port;
          uint16_t dst_port;
        } ip4;
        struct {  /* for TCP/UDP over IPv6, len = 36 */
          uint8_t  src_addr[16];
          uint8_t  dst_addr[16];
          uint16_t src_port;
          uint16_t dst_port;
        } ip6;
        struct {  /* for AF_UNIX sockets, len = 216 */
          uint8_t src_addr[108];
          uint8_t dst_addr[108];
        } unx;
      } addr;
    } v2;
  } header_;

  std::string ip_address_;
  uint16_t port_ = 0;
};

}

#endif //LIBC1_NET_SRC_PROXYPROTOCOL_H_
