/* Copyright 2022 Sensaru GmbH */

#ifndef LIBC1_NET_SRC_SOCKET_H_
#define LIBC1_NET_SRC_SOCKET_H_

#include <memory>
#include <atomic>

#include <unistd.h>
#include <sys/socket.h>

namespace C1Net {

class Socket {
 public:
  Socket() = default;
  explicit Socket(int socket);
  ~Socket();

  int GetHandle();

  bool IsValid();

  /**
   * Cleans up and reinits the socket with a new socket descriptor. This avoids creation of a new socket object.
   *
   * @param socket
   */
  void Reinit(int socket);

  /**
   * Shut down the socket. This causes all functions using the socket (poll, read, ...) to return immediately. After a
   * call to Shutdown() IsValid() always returns `false`.
   */
  void Shutdown();
 private:
  std::atomic_bool is_shutdown_{false};
  std::atomic_int socket_{-1};
};

typedef std::shared_ptr<Socket> PSocket;

}

#endif //LIBC1_NET_SRC_SOCKET_HPP_
