/* Copyright 2022 Sensaru GmbH */

#ifndef LIBC1_NET_SRC_STATISTICS_H_
#define LIBC1_NET_SRC_STATISTICS_H_

namespace C1Net {

class Statistics {
 public:
  Statistics() = delete;

  /**
   * Calculates the exponential moving average per time for performance metrics.
   *
   * @param interval The time since the last measurements. The time unit doesn't matter as long as it is the same unit as for `period`.
   * @param period The time to average over (e. g. 60 seconds or 15 minutes). The time unit doesn't matter as long as it is the same as for `interval`.
   * @param metric The current performance measurement.
   * @param last_average The last output of this method. Pass `0` when this is the first call.
   * @return Returns the average over the given time period.
   */
  static double MetricExponentialMovingAverage(double interval, double period, double metric, double last_average);
};

}

#endif //LIBC1_NET_SRC_STATISTICS_H_
