/* Copyright 2022 Sensaru GmbH */

#ifndef LIBC1_NET_SRC_TLSCOMMON_H_
#define LIBC1_NET_SRC_TLSCOMMON_H_

#include <memory>
#include <string>
#include <shared_mutex>
#include <atomic>

#include <gnutls/gnutls.h>

namespace C1Net {

class TlsSession {
 public:
  explicit TlsSession(unsigned int flags);
  ~TlsSession();

  void Reinit();

  bool IsValid();

  gnutls_session_t GetHandle();

  static std::string GetGnuTlsCertificateVerificationError(uint32_t error_code);
 private:
  unsigned int flags_ = 0;
  std::atomic<gnutls_session_t> session_{nullptr};
};

typedef std::shared_ptr<TlsSession> PTlsSession;

class TlsPriorityCache {
 public:
  TlsPriorityCache();
  ~TlsPriorityCache();

  bool IsValid();

  gnutls_priority_t GetHandle();
 private:
  gnutls_priority_t priority_cache_ = nullptr;
};

typedef std::shared_ptr<TlsPriorityCache> PTlsPriorityCache;

}

#endif //LIBC1_NET_SRC_TLSCOMMON_H_
